/* nstat.c
 * Demonstrates use of an undocumented IOCTL call to get the
 * NETBIOS name given an IP.
 *
 * basically replaces the functionality of an "nbtstat -A <ip address>" command.
 *
 *
 *  Copyright (C) 1999 Greg Hoglund
 *
 ********************************************************************
 * Jan 1, 1999 - Juggler of ChAoS
 * inital release
 *
 * Jan 10, 1999 - Thufir
 * Revision notes:
 * Stuffed things copied from MS headers into header file (can't GPL MS's stuff)
 * Added proper declaration for NtDeviceIoControlFile
 * Turned into proper UNICODE - deleted ascii macro stuff
 * reformatted things for better readability - so I'm a format nazi, sue me
 * Added handling for multiple devices
 * Fixed it to print the MAC address
 * Misc output formatting
 * for some weird reason, NB name tables sometimes have unprintable chars
 * - these are now converted to '.'
 *
 ********************************************************************
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#define UNICODE
#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include <malloc.h>
#include "nstat.h"

void dumpNames(const char *outbuf);

//leave these as globals - makes things a little cleaner
NTSTATUS (__stdcall *fNtCreateFile)(
				PHANDLE FileHandle,
				ACCESS_MASK DesiredAccess,
				POBJECT_ATTRIBUTES ObjectAttributes,
				PIO_STATUS_BLOCK IoStatusBlock,
				PLARGE_INTEGER AllocationSize,	
				ULONG FileAttributes,
				ULONG ShareAccess,
				ULONG CreateDisposition,
				ULONG CreateOptions,
				PVOID EaBuffer, 
				ULONG EaLength);

NTSTATUS (__stdcall *fNtDeviceIoControlFile)(
				HANDLE FileHandle,
				HANDLE Event, //optional
				PIO_APC_ROUTINE ApcRoutine, //optional
				PVOID ApcContext, //optional
				PVOID pRequest,	
				ULONG IoControlCode,
				PVOID InputBuffer, //optional
 				ULONG InputBufferLength,
				PVOID OutputBuffer,
				DWORD OutputBufferSize);

BOOL GetFunctionPtrs(void)
{
	HMODULE hMod;

	//call this once, check return once
	hMod = GetModuleHandle(L"ntdll.dll");
 
	if(hMod == NULL)
	{
		fprintf(stderr, "Cannot get handle of ntdll.dll\n");
		return FALSE;
	}
	
	fNtCreateFile = (void *) GetProcAddress(hMod, "NtCreateFile");
	
	if(fNtCreateFile == NULL)
	{
		fputs("Could not find entry point in NTDLL.DLL\n", stderr);
		return FALSE;
	}

	fNtDeviceIoControlFile = (void *) GetProcAddress(hMod, "NtDeviceIoControlFile");

	if(fNtDeviceIoControlFile == NULL) 
	{
		fputs("Could not find entry point in NTDLL.DLL\n", stderr);
		return FALSE;
	}
	return TRUE;
}

BOOL OpenDeviceHandles(HANDLE* hDev, BYTE* NumHandles)
{
	HKEY aKey;
	WCHAR* Linkages = NULL;
	WCHAR* Tmp;
	DWORD Len, BufSize;
	DWORD ret;
	BOOL retval = FALSE;
	UNICODE_STRING aSS;
	OBJECT_ATTRIBUTES anObjAttrib;
	IO_STATUS_BLOCK	aReturnCode;
	BYTE NumDev = 0;

	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
				    L"system\\currentcontrolset\\services\\netbt\\linkage", 
				    0,
				    KEY_READ,
				    &aKey) != ERROR_SUCCESS)
	{
		fprintf(stderr, "Cannot open NetBt key\n");
		return FALSE;
	}

	//try a 1k buffer to begin with - should be more than enough
	Linkages = (WCHAR*)GlobalAlloc(GPTR, 1024);
	if(Linkages == NULL)
	{
		fprintf(stderr, "Can't allocate 1k!!! Aeeeieieieee!!!\n");
		RegCloseKey(aKey);
		return FALSE;
	}

	BufSize = 1024;

	//there could be lots of these - we just don't know
	//so we'll set up the function to get them all
	ret = RegQueryValueEx(aKey,
						  L"Export",
						  NULL,
						  NULL,
						  (BYTE*)Linkages,
						  &BufSize);
	
	switch(ret)
	{
	case ERROR_MORE_DATA:
		//retry
		GlobalFree(Linkages);
		Linkages = (WCHAR*)GlobalAlloc(GPTR, BufSize);
		if(Linkages == NULL)
		{
			fprintf(stderr, "Can't allocate %d bytes! Die!\n", BufSize);
			break;
		}

		if(RegQueryValueEx(aKey,
					       L"Export",
						   NULL,
						   NULL,
						   (BYTE*)Linkages,
						   &BufSize) != ERROR_SUCCESS)
		{
			fprintf(stderr, "cannot get exports\n");
			break;
		}
		//else fall through
	case ERROR_SUCCESS:
		//we got the data

		/* InitializeObjectAttributes */
		memset((void *)&anObjAttrib, 0, sizeof(OBJECT_ATTRIBUTES));
		anObjAttrib.Length = sizeof(OBJECT_ATTRIBUTES);
		anObjAttrib.Attributes = 0x40; /* sniffed this value */

		Tmp = Linkages;
		while((Len = wcslen(Tmp)) && NumDev < 64)
		{
			aSS.Length = wcslen(Tmp)*sizeof(WCHAR);
			aSS.MaximumLength = aSS.Length + sizeof(WCHAR);
			aSS.Buffer = Tmp;
			anObjAttrib.ObjectName = &aSS;

			ret = fNtCreateFile(&hDev[NumDev],
								0x00100003, /* snooped this too */
								&anObjAttrib,
								&aReturnCode,
								0,	/* optional */
								0x80, /* file attrib, snooped this */
								0x3,
								0x3,
								0,
								NULL,
								0);
			if(ret != STATUS_SUCCESS)
			{
				fprintf(stderr, "NtCreateFile error %d\n", GetLastError());
				fprintf(stderr, "Can't open %S\n", Tmp);
				//bail out of while
				NumDev = 64;
				break;
			}

			NumDev++;
			Tmp += Len + 1;
		}

		if(NumDev > 0)
			retval = TRUE;
		break;
	default:
		//some weird error we don't expect
		break;
	}

	//cleanup
	if(Linkages != NULL)
		GlobalFree(Linkages);

	RegCloseKey(aKey);
	*NumHandles = NumDev;
	return retval;
}


int main( int argc, char *argv[])
{
	//this is arbitrary - there shouldn't be more than this,
	//and WaitForMultipleObjects gets to be a pain if there are, so...
	HANDLE hDevice[64];
	NTSTATUS ret;
	WCHAR *aUs = 0;
	BYTE i, NumHandles;
	char* outbuf[64]; 
	TDI_REQUEST_UNKNOWN2 aTdi2;

	if(argc != 2)
	{
		fprintf(stdout, "Usage: %s <ip address>\n", argv[0]);
		return 0;
	}

	if(!GetFunctionPtrs())
		return -1;

	//initialize our handles
	ZeroMemory(hDevice, 64*sizeof(HANDLE));

	if(!OpenDeviceHandles(hDevice, &NumHandles))
	{
		return -1;
	}

	for(i = 0; i < NumHandles; i++)
	{
		outbuf[i] = GlobalAlloc(GPTR, 600);
		if(outbuf[i] == NULL)
		{
			fprintf(stderr, "Can't alloc! AiEiEiEiE!!!\n");
			break;
		}
		/* this is hax0red sushi */
		aTdi2.Request.Handle.AddressHandle = (void *) htonl(inet_addr(argv[1]));
		aTdi2.Request.RequestNotifyObject = (void *) 1;
		aTdi2.Request.RequestContext = (void *) 0x110012;
		aTdi2.Request.TdiStatus = 0x2a0000;
		aTdi2.a = 0;
		aTdi2.b = 0;
		aTdi2.c = 0;
		aTdi2.d = 0;

		/* outbuf2 should be filled with the nbtstat data */
		ret = fNtDeviceIoControlFile(	hDevice[i],
										0,
										0,
										0,
										&aTdi2, // ???
										IOCTL_TDI_UNKNOWN2, //0x2100A6, // code
										&aTdi2,
										0x20,
										outbuf[i],
										0x258
										);
		if(ret != STATUS_PENDING && ret != STATUS_SUCCESS)
		{
			fprintf(stderr, "Weird error\n");
			break;
		}
	}

	ret = WaitForMultipleObjects(NumHandles, hDevice, FALSE, 30000);

	if(ret == WAIT_TIMEOUT)
	{
		fputs("Timed out waiting for response...\n", stderr);
	}
	else
	{
		dumpNames(outbuf[ret - WAIT_OBJECT_0]);
	}

	for(i = 0; i < NumHandles; i++)
	{
		GlobalFree(outbuf[i]);
		CloseHandle(hDevice[i]);
	}

	return 0;
}


#define NB_GROUP 0x80

void dumpNames(const char *outbuf)
{
	struct netbiosName *aNameP;
	char numEntries;
	char temp[16];
	BYTE i, x;

	//MAC addr is 1st 6 bytes

	fprintf(stdout, "MAC address:\n");
	
	for(i = 0; i < 6; i++)
	{
		x = outbuf[i];
		fprintf(stdout, "%02X", x);
		if(i < 5)
			fprintf(stdout, "-");
	}
	fprintf(stdout, "\n\n");

	outbuf += 0x3a;  /* name table starts 58 bytes in */
	numEntries = *outbuf;
	outbuf += 2;

	temp[15] = '\0';

	while(numEntries--)
	{
		aNameP = (struct netbiosName *)outbuf;
		strncpy(temp, aNameP->name, 15);
		//sanitize output
		for(i = 0; i < 15; i++)
		{
			if(!isprint(temp[i]))
				temp[i] = '.';
		}
		if(aNameP->registered & NB_GROUP)
			fprintf(stdout, "%16s<%02x> %-10s\n", temp, aNameP->type, "GROUP");
		else
			fprintf(stdout, "%16s<%02x> %-10s\n", temp, aNameP->type, "UNIQUE");

		outbuf += sizeof(struct netbiosName);
	}
}


