# This is the Mentat file
# (c)1999 Greg Hoglund
#
# Give it a spin.  This file is intended to control all of the logic for the
# RogueX scanner.  More functions will be added as time goes on.
#


$gLocalNode;
$gCurrentNode;

sub OnCommand{
	my $theCommand = shift;
	
	chop($theCommand);
	#DebugOut("Perl got the command $theCommand\n");

	if($theCommand =~ m/^scan (\S+)/)
	{
		my $target = $1;
		if($target =~ m/[0-9\.]/){
			DisplayWindowText("\n\n I'm takin' a look at\n$target\n\n");
			ManualScan($target);
		}
		else{
			DisplayWindowText("\n\n The target $target \ndoesn't look like an \nIP address\n");
		}
	}
	elsif($theCommand =~ m/^add (\S+) (.*)/)
	{
		my $key = $1;
		my $data = $2;
		if($key =~ m/[0-9\.]/){
			DisplayWindowText("\n\n I'm adding\nKey: $key\nData: $data\n");
			AddDataNode($key, $data);
		}
		else{
			DisplayWindowText("\n\n The key $key \ndoesn't look like an \nIP address\n");
		}
	}
	elsif($theCommand =~ m/^look (\S+)/)
	{
		my $key = $1;
		if($key =~ m/[0-9\.]/){
			DisplayWindowText("\n\n I'm looking at\n$key\n\n");
			SetCurrNode($key);
			$gCurrentNode = $key;
		}
		else{
			DisplayWindowText("\n\n The key $key \ndoesn't look like an \nIP address\n");
		}
	}
	elsif($theCommand =~ m/^list (\S+)/)
	{
		my $key = $1;
		ListNode($key);
	}
	elsif($theCommand =~ m/^list/)
	{
		ListNode($gCurrentNode);
	}		
	elsif($theCommand =~ m/^hello/gi)
	{
		DebugOut("Perl got the command $theCommand\n");
		DisplayWindowText("\n\n Hello.\n");
	}
	return(0,0);
}

sub ManualScan {
	my $target = shift;

	$did_echo = 0;

	@result = `ping $target`;
	foreach(@result){
		if(m/timed out/gi){
			DisplayWindowText("The ping request timed out\n");
		}
		elsif(m/reply/gi){
			#DisplayWindowText($_);
			$did_echo = 1;
		}
	}
	if($did_echo){
		AddDataNode($target, "");  # OK to add a blank entry.. creates a new globe for the IP
		SetCurrNode($target); # set the viewport to the current IP
		DisplayWindowText("\n\nThe target is responding\n");
		UDPScan32($target); # of course (builds UDP data)
		SYNScan32($target); # of course (builds TCP data)
	}
}

# This section is responsible for turning on/off the LED buttons
# on the display.  Idea: You can add hooks for any of the options
# and run your own custom code as well.

sub OnMouseClick {
	my $x = shift;
	my $y = shift;
	DebugOut("detected click at $x, $y");
	if( ($y > 27)
	    &&
		($y < 36))
	{
		if( ($x > 160)
		    &&
			($x < 169))
		{
			ToggleState("BANNER");
			ClearWindowText();
			DisplayWindowText( "\n\nGather banners.\nA typical information source.\nUses service banners.\n");
		}
		elsif( ($x > 169)
		    &&
			($x < 177))
		{
			ToggleState("FORCE");
			ClearWindowText();
			DisplayWindowText( "\n\nForce scan.\nScan IP regardless of responses.\nTakes a long time.\n");
		}
		elsif( ($x > 177)
		    &&
			($x < 189))
		{
			ToggleState("PROBE");
			ClearWindowText();
			DisplayWindowText( "\n\nProbe Networks.\nAutomatic scanning will\ndiscover all hosts between\nyou and the target.  Exploratory.\n");
		}
	}
	
	if($x < 192)
	{
		if($x > 138)
		{
			if( ($y > 45) 
				&&
				($y < 59))
			{	
				ToggleState("SYNC");
				ClearWindowText();
				DisplayWindowText( "\n\nSYNC SCANNING\nA typical high speed scan.\nUses SYN flag on TCP packets.\n");
			}
			elsif( ($y > 59) 
			       &&
				   ($y < 73))
			{
				ToggleState("ACK");
				ClearWindowText();
				DisplayWindowText( "\n\nACK SCANNING\nA stealth mode scan.\nUses ACK flag on TCP packets.\n");
			}
			elsif( ($y > 73 )
			       &&
				   ($y < 87 ))
			{
				ToggleState("FIN");
				ClearWindowText();
				DisplayWindowText( "\n\nFIN SCANNING\nA stealth mode scan.\nUses FIN flag on TCP packets.\n");
			}
			elsif( ($y > 87 )
			        &&
				   ($y < 101))
			{
				ToggleState("URECV");
				ClearWindowText();
				DisplayWindowText( "\n\nUDP RecvFrom SCANNING\nA typical connection scan.\nFrobs UDP ports for response.\n");
			}
			elsif( ($y > 101)
			       &&
				   ($y < 115))
			{
				ToggleState("UDSTU");
				ClearWindowText();
				DisplayWindowText( "\n\nUDP Destination Unreachable\nA more efficient UDP scan.\nListens for ICMP \nDestination Port Unreachable \npackets.\n");
			}
		}
	}
}

sub OnPacket{
	# this is called for every IP packet
	my $data = shift;
	my $len = shift;
	DebugOut(" got packet, length $len");

	# of course, you can now respond to the packet as you see fit
	# lets break down some packets (doesnt account for IP options, 
	# so in a future release I must fix the data handling to account
	# for various packet types & header lengths..)

	($ES1, $ES2, $ES3, $ES4, $ES5, $ES6, 
	 $ED1, $ED2, $ED3, $ED4, $ED5, $ED6,
	 $Ether_type,
	 $IPData_vlen,
	 $IPData_stype, 
	 $IPData_tlen,
	 $IPData_id,
	 $IPData_flagfrag, 
	 $IPData_ttl,   
	 $IPData_proto,
	 $IPData_checksum,
	 $IPData_srcaddr,  
	 $IPData_destaddr,
	 $TCPData_srcport,
	 $TCPData_destport,
	 $TCPData_seqnum,	
	 $TCPData_ack,	
	 $TCPData_off_res_flags,	
	 $TCPData_window,
	 $TCPData_checksum,	
	 $TCPData_urgent) = unpack("CCCCCCCCCCCCnCCnnnCCnNNnnNNnnnn", $data);

	DebugOut("detected ether type $Ether_type");
	DebugOut("IP breakdown proto $IPData_proto");
	if(6 == $IPData_proto)
	{
		DebugOut("From port $TCPData_srcport to $TCPData_destport flags $TCPData_off_res_flags");
	}
	return(0,0);	
}

# debugging here
sub nice_number {
	my $num = shift;
	1 while ($num =~ s/(.*\d)(\d\d\d)/$1,$2/g);
	$num;
}

# called on system startup
sub test_nice{
	DebugOut("Hello World");
	AddDefaultTool(1000);
	DisplayWindowText("Hey man, I'm online...\n");

	SendRawPacket("FF FF FF FF FF FF"); #testing

	#just testing here
	@result = `route print`;
	foreach(@result){
		m/\s*([\d\.]*)\s*([\d\.]*)\s*([\d\.]*)\s*([\d\.]*)\s*([\d\.]*).*/g;
		my $dest = $1;
		my $interface = $4;
		DebugOut("$1 $2 $3 $4");
		if( $dest =~ m/255\.255\.255\.255/g){
			DisplayWindowText("\n\n\n\nAdding local host as\n$interface\n");
			AddDataNode($interface, "LOCALHOST");
			$gLocalNode = $interface;
			$gCurrentNode = $interface;
		}
	}
	
	return(1000, 2000);
}
